function HTML_spec(str,   s){
  s = str;
  gsub(/\&/,"\\\\\\&amp;",s);
  gsub(/\"/,"\\\\\\&quot;",s);
  gsub(/</,"\\\\\\&lt;",s);
  gsub(/>/,"\\\\\\&gt;",s);
  return s;
}

#---------------- csv_get_rec -----------------
#str -> "f1","f2", ... ,"fn"
#arr_res[1]=f1; arr_res[2]=f2; ... arr_res[n]=fn;
#return fields number   
#----------------------------------------------
function csv_get_rec(str,arr_res,    n){
  n = split(str,arr_res, "\",\"");
  gsub(/^"/,"",arr_res[1]);
  gsub(/"$/,"",arr_res[n]);
  return n;
}

#---------------- csv_make_rec -----------------
#arr_fld[1]=f1; arr_fld[2]=f2; ... arr_res[n_fld]=fn_fld;
#return -> "f1","f2", ... ,"fn_fld"
#-----------------------------------------------
function csv_make_rec(arr_fld,n_fld,   i,s){
  s = "\"" arr_fld[1] "\"";
  for(i=2; i<=n_fld; i++)
    s = s "," "\"" arr_fld[i] "\"";
  return s;
}

#---------------- tbl_new -----------------
#            create a new table
#a_data[key] = record (now empty)
#s_key - key of table
#s_fld - fields list (f1#f2#...#fn)
#a_ind - for record numbers and serv info
#a_ind["rec_total"] - total records count (now 0)
#a_ind["fld_list"]  = s_fld
#a_ind["fld_total"] - total fields count
#a_ind["key_index"] - number of key position in fields list
#a_ind["f"] - number of field "f" position in fields list
#return 1 - no key in fields list
#       0 - OK
#-------------------------------------------
function tbl_new(a_ind, a_data, s_fld, s_key,    tmp_a,i,n,k){
  delete a_ind;
  delete a_data;
  a_ind["rec_total"] = 0;
  a_ind["fld_list"] = s_fld;
  n = split(s_fld,tmp_a,"#");
  if(n==0) return 1;
  for(i=1; i<=n; i++)
    a_ind[tmp_a[i]] = i;
  a_ind["fld_total"] = n;
  k = 0;
  for(i=1; i<=n; i++)
    if(s_key == tmp_a[i]) k = i;
  if(k==0) return 1;
  a_ind["key_index"] = k;
  return 0;
}

#---------------- tbl_get_rec -----------------
#            get a record from the table
#s_key - key of record
#a_val - array of fields values
#a_data[s_key] = string of fields values
#return 1 - record with key "s_key" new, only key value put into a_val
#       0 - record with key "s_key" existed, fields values put into a_val
#----------------------------------------------
function tbl_get_rec(a_ind, a_data, s_key, a_val,   i){
  if(s_key in a_data){
    csv_get_rec(a_data[s_key],a_val);
    return 0;
  }
  delete a_val;
  for(i=1; i<=a_ind["fld_total"]; i++) a_val[i] = "";
  a_val[a_ind["key_index"]] = s_key;
  return 1;
}


#---------------- tbl_put_rec -----------------
#            put a record into the table
#s_key - key of record
#a_val - array of fields values
#a_data[s_key] = string of fields values
#return 1 - record with key "s_key" inserted
#       0 - record with key "s_key" existed and updated
#----------------------------------------------
function tbl_put_rec(a_ind, a_data, s_key, a_val,   s_data){
  a_val[a_ind["key_index"]] = s_key;
  s_data = csv_make_rec(a_val,a_ind["fld_total"]);
  if(s_key in a_data){
    a_data[s_key] = s_data;
    return 0;
  }
  a_data[s_key] = s_data;
  a_ind[++a_ind["rec_total"]] = s_key;
  return 1;
}

#---------------- tbl_sort ---------------------
#        sort information in a table
#
#s_fld - fields list in string (separator -"#")
#s_stp - sort types in string (separator -"#")
#stp=="DESC" - descending sort
#stp!="DESC" - ascending sort
#return 1 - nothing to sort
#       0 - OK
#-----------------------------------------------
function tbl_sort(a_ind,a_data,s_fld,s_stp,
                  pri,a_fld,a_stp,k,n,ai,aj,nr,i,j,si,sj,v){
  n = split(s_fld,a_fld,"#");
  split(s_stp,a_stp,"#");
  nr = a_ind["rec_total"];
  if(nr<=1) return 1;
  for(i=1; i<nr; i++)
    for(j=i+1; j<=nr; j++){
      tbl_get_rec(a_ind,a_data,a_ind[i],ai);
      tbl_get_rec(a_ind,a_data,a_ind[j],aj);
      pri = 0;
      for(k=1; k<=n; k++){
        si = ai[a_ind[a_fld[k]]];
        sj = aj[a_ind[a_fld[k]]];
        if(si == sj) continue;
        if(((a_stp[k]=="DESC") && (si<sj)) ||
           ((a_stp[k]!="DESC") && (si>sj)) )
          pri = 1;
        break;
      }
      if(pri){
        v = a_ind[i];
        a_ind[i] = a_ind[j];
        a_ind[j] = v;
      }
    }
  return 0;
}

#---------------- tbl_debug_print -----------------
#            debug print a table to file
#fn - file name (if fn=="" - print to output stream)
#return - nothing
#--------------------------------------------------
function tbl_debug_print(a_ind, a_data,fn,   i){
  print "---------------------";
  for(i in a_ind)
    if(i !~ /^[0-9]/)
      if (fn != "") printf("a_ind[%s]=%s\n",i,a_ind[i]) > fn
               else printf("a_ind[%s]=%s\n",i,a_ind[i]);
  if (fn != "") print "-------------------------"  > fn
           else print "-------------------------";
  for(i=1; i<=a_ind["rec_total"]; i++)
    if (fn != "") printf("[%s]=%s ->%s\n",i,a_ind[i],a_data[a_ind[i]]) > fn
             else printf("[%s]=%s ->%s\n",i,a_ind[i],a_data[a_ind[i]]);
}

#--------------------- read_ptn  ------------------
#              read pattern for report
#fn - file name
#return -1 if error
#--------------------------------------------------
function read_ptn(fn,  i,r,s){
  ptn_count = 0;
  while(r = getline s < fn){
    if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
    if(s ~ /^<!##########/)
      ptn_count++;
    else
      ptn_src[ptn_count] = ptn_src[ptn_count] s "\n";
  }
  close(fn);
  ptn_count++;
  for(i=0; i<ptn_count; i++)
    ptn_cur[i] = ptn_src[i];
}

#--------------------- read_strings  ------------------
#              read strings list from file
#fn - file name
#srt_arr - array for strings
#return -1 if error
#------------------------------------------------------
function read_strings(fn,srt_arr,    s_key,s_val,r){
  while(r = getline s_key < fn){
    if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
    r = getline s_val < fn
    srt_arr[s_key] = s_val;
  }
  return 0;
}

#-------------------- make_html_name  -----------------
#------------------------------------------------------
function make_html_name(s,   fn){
  fn = s;
  gsub(/ /,"_",fn);
  fn = fn ".html";
  return fn;
}

BEGIN{
  read_strings("___str.___", GB);
  read_ptn("part.__h");
  pref = GB["PR"];
}

NR == 1{
  next;
}

#"id","TIME","userid","ip","course","module","cmid","action","url","info"
#  1     2       3      4      5        6       7       8       9     10
{
  csv_get_rec($0,arr_line);
  ac = arr_line[8];
  tmp[ac]++;
  fn = ac;
  fn = pref make_html_name(fn);
  if(tmp[ac] == 1){
      ptn_cur[0] = ptn_src[0];
      gsub(/==ID==/,GB["ID"],ptn_cur[0]);
      gsub(/==NM==/,GB["NM"],ptn_cur[0]);
      gsub(/==AC==/,ac,ptn_cur[0]);
      printf(ptn_cur[0]) > fn;
  }
#  print $0 > fn;
  ptn_cur[1] = ptn_src[1];
  gsub(/==N==/,tmp[ac],ptn_cur[1]);
  gsub(/==F1==/,arr_line[1],ptn_cur[1]);
  gsub(/==F2==/,arr_line[2],ptn_cur[1]);
  gsub(/==F3==/,arr_line[3],ptn_cur[1]);
  gsub(/==F4==/,arr_line[4],ptn_cur[1]);
  gsub(/==F5==/,arr_line[5],ptn_cur[1]);
  gsub(/==F6==/,arr_line[6],ptn_cur[1]);
  gsub(/==F7==/,arr_line[7],ptn_cur[1]);
  gsub(/==F8==/,arr_line[8],ptn_cur[1]);
  gsub(/==F9==/,HTML_spec(arr_line[9]),ptn_cur[1]);
  gsub(/==F10==/,arr_line[10],ptn_cur[1]);
  printf(ptn_cur[1]) > fn;
}

END{
  tbl_new(indAC, dataAC, "KEY#NUMB", "KEY");
  for (s in tmp){
#    print s " -> " tmp[s];
    fn = pref make_html_name(s);
    printf(ptn_src[2]) > fn;
    tmp_dt[1] = s;
    tmp_dt[2] = tmp[s];
    tbl_put_rec(indAC, dataAC, s, tmp_dt);
  }
  tbl_sort(indAC,dataAC,"KEY","ASC");

  delete ptn_src;
  delete ptn_cur;
  read_ptn("ind.__h");
  fn = pref GB["ID"] ".html";
  ptn_cur[0] = ptn_src[0];
  gsub(/==ID==/,GB["ID"],ptn_cur[0]);
  gsub(/==NM==/,GB["NM"],ptn_cur[0]);
  printf(ptn_cur[0]) > fn;
  for(i=1; i<=indAC["rec_total"]; i++){
    csv_get_rec(dataAC[indAC[i]],arr_line);
    rf = make_html_name(arr_line[1]);
    ptn_cur[1] = ptn_src[1];
    gsub(/==N==/,i,ptn_cur[1]);
    gsub(/==F1==/,arr_line[1],ptn_cur[1]);
    gsub(/==F2==/,arr_line[2],ptn_cur[1]);
    gsub(/==RF==/,rf,ptn_cur[1]);
    printf(ptn_cur[1]) > fn;
  }
  printf(ptn_cur[2]) > fn;
  tbl_debug_print(indAC,dataAC);
}
